/**
 * Copyright (C) 2011 Agrosense <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.crop.model;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * Storage object for different crops
 * Directly adding crops to the croplist is not possible to prevent concurrent modification exceptions. 
 * Crops should be added/removed through the set- add- and remove- methods in this class.
 * The property-change firing in the set- add- and remove methods will cause users of this croplist 
 * to get a new unmodifiable copy of the list of crops through the getCrops() method.
 * 
 * @author Merijn Zengers & Gerben Feenstra
 */
public class CropsStorage extends CropBean implements Serializable{
    private List<Crop> crops = Collections.synchronizedList(new ArrayList<Crop>());
    
    public static final String PROP_CROP = "crops";

    /**
     * Returns unmodifiable copy of croplist to prevent concurrent modification exceptions. (See class comment)
*    * @return Unmodifiable copy of croplist
     */
    public List<Crop> getCrops() {        
        List<Crop> copyCrops = new ArrayList<>(crops);
        return Collections.unmodifiableList(copyCrops);
    }

    /**
     * Do not change param to a generic variant, this is needed for undo redo! (Won't work with generics)
     * TODO: Find a solution to enforce a generic List of crops
     * @param crops A list of crops - WARNING: Only crops!
     */
    public void setCrops(ArrayList crops) {
        List<Crop> old = this.crops;
        this.crops = crops;
        getPropertyChangeSupport().firePropertyChange(PROP_CROP, old, crops);
    }

    /**
     * Add a crop to this cropStorage
     * @param crop Crop to add
     */
    public void addCrop(Crop crop) {
        List<Crop> old = new ArrayList<>(this.crops); 
        this.crops.add(crop);
        getPropertyChangeSupport().firePropertyChange(PROP_CROP, null, crop);
        if(crop.getSource().equals(Crop.USER_INPUT)){
            fireUndoableEvent(PROP_CROP, this, old, this.crops);
        }
    }
    
    /**
     * Remove a crop from this cropStorage
     * @param crop Crop to remove
     */
    public void removeCrop(Crop crop){
        List<Crop> old = new ArrayList<>(this.crops); 
        this.crops.remove(crop);
        getPropertyChangeSupport().firePropertyChange(PROP_CROP, crop , null);
        fireUndoableEvent(PROP_CROP, this, old, this.crops);
    }
}
